# Device::Tektronix - Perl Module to interface and control a Tektronix
#                  THS7xx handheld oscilloscope.  Developed and tested
#                  with a THS720A
# (c) 2003 - Max Baker - See tail of file for Licesense
package Device::Tektronix;

use strict;
use warnings;
require Exporter;

use vars qw/$VERSION @ISA %EXPORT_TAGS @EXPORT_OK @EXPORT
            $SerialClass @Baud/;

# Package Globals
$VERSION = '0.1';

@ISA = qw(Exporter);
%EXPORT_TAGS = ( 'all' => [ qw() ] );
@EXPORT_OK   = ( @{ $EXPORT_TAGS{'all'} } );
@EXPORT      = qw( );

# Possible Baud rates
@Baud = qw/300 600 1200 2400 4800 9600 19200 38400/;

# Load the Serial Port Module (win or unix?)
if ($^O =~ /win32/i){
    eval "use Win32::SerialPort;";
    if ($@) {
        print "Please install Win32::SerialPort.  Using console for output.\n";
        $SerialClass = 'console';
    } else {
        $SerialClass = 'Win32::SerialPort';
    }
} else {
    eval "use Device::SerialPort;";
    if ($@){
        print "Tektronx::THS Requires either Win32::SerialPort or Device::SerialPort installed. \n Using Console for output.\n";
        $SerialClass = 'console';
    } else {
        $SerialClass = 'Device::SerialPort';
    }
}

=head1 NAME

Device::Tektronix 0.1 - Command interface for Tektronix THS Handheld Oscilloscopes

=head1 SYNOPSIS

 my $tek = new Device::Tektronix(debug => 1,
                              port  => 'com1',
                              baud  => 38400
                             ) or die;

 $tek->connect or die $tek->error;

 $oldsource = $tek->waveform->source;
 $tek->waveform->encoding('ulsb');   # signed, MSB first
 $tek->waveform->source("ch2");
 $tek->waveform->start(1);
 $tek->waveform->stop(2049);
 $tek->waveform->width(8);

 $tek->trigger->auto_set;

 $tek->channel->bandwidth($channel,'full');

 $scale = $tek->time->scale;  

 $data = $tek->waveform->get;

See tek-fft included with the distribution of this Perl
Module for a full and complete example of the API.

=head1 DESCRIPTION

THS.pm is an object-oriented interface to the Tektronix
THS700 series handheld oscilloscopes.  It uses either Win32::SerialPort
or Device::SerialPort for communications under Unix and Windows.

Advanced features include waveform datalogging, and extended DMM
logging.

The full command set is not implemented yet, but should
be done by version 0.2.

This project lives at http://warped.org/tek.

=head1 AUTHOR

Max Baker

=head1 Methods

=head2 Constructor and Fundamental Methods

=over

=item new([args])

New will accept a few arguments : 

    debug     - Print Copious debug info to stdout
    port      - Specify the com port (com1,com2)
    baud      - BAUD rate to use for scope (default is 9600)
                Make sure your scope matches, and speed it up to 38,400!
    flaghard  - RTS/CTS  or Hardware Flow Control (0 or 1), default on
    flagsoft  - Xon/Xoff or Software Flow Control (0 or 1)
    parity    - Parity Bit for RS232  (0,1)
    stopbits  - Stop Bits for RS232  (1)
    retries   - Number of times to retry a command before failing. (10)
    timeout   - Timeout in Seconds for each retry on serial port  (10)
    serialize - Boolean.  Use to turn off BUSY? checking.  Use with caution.

Don't forget to $tek->connect()!

This module is designed to work in silent mode, so that you can build an
application around it.   Any command will return undef if there is an error 
or failure.  Check the error() method of your object if you get an undef,
or turn on debug.

=cut
# TODO - Make a 'Verbose Mode' with half the debug.
sub new {
    my $proto = shift;
    my %args = @_;
    my $class = ref($proto) || $proto;
    my $self  = {};

    $self->{Debug}    = 1 if (defined $args{debug} and $args{debug});
    $self->{Port}     = $args{port} || 'com1';
    $self->{Baud}     = $args{baud} || '9600';
    $self->{FlagHard} = $args{flaghard} || 1;
    $self->{FlagSoft} = $args{flagsoft} || 0;
    $self->{Parity}   = $args{parity}   || 0;
    $self->{Stopbits} = $args{stopbits} || 1;
    $self->{Autobaud} = defined $args{autobaud} ? $args{autobaud} : 1;
    $self->{Retries}  = $args{retries} || 10;
    $self->{Timeout}  = $args{timeout} || 10;
    $self->{Busywait} = defined $args{serialize} ? $args{serialize} : 1;

    # Add Subclasses on to us 
    #   This kludge is called Java-like inherited interfaces
    #   Purely for amusment purposes.  I don't like Java.
    foreach my $interface (qw/Aquire Alias Diag Cursor Display Print
                              Harmonics Time Channel Measure Serial
                              Trigger Waveform DMM /) 
    {
        no strict 'refs';
        my $class = "Device::Tektronix::$interface";
        my $new   = "${class}::new";
        # Store the subobject by its name
        $self->{$interface} = &$new($class,$self);
        # Add an accessor method for each subclass
        my $accessor = lc($interface);
        *$accessor  = sub { shift->{$interface}; }
    }
    bless ($self, $class);
    return $self;
}

=item $tek->connect()

Connects your object to the scope through the serial port.
Note, changes some settings.

=cut
sub connect {
    my $self = shift;

    # Serial Port Settings
    my $port = $self->{Port};
    my $baud = $self->{Baud};
    my $hwflow = $self->{FlagHard};
    my $swflow = $self->{FlagSoft};
    my $parity = $self->{Parity};
    my $stopbits = $self->{Stopbits};
    my $autobaud = $self->{Autobaud}; 
    
    my $portobj = undef;
    {
        no strict 'refs';
        my $new = "${SerialClass}::new";
        # 3rd arg is 'quiet'
        $portobj = &$new($SerialClass,$port,$self->debug)
            || $self->throw("Can't open $port: $^E\n");
    }

    $portobj->databits(8);
    $portobj->parity($parity);
    $portobj->stopbits($stopbits);
    $portobj->baudrate($baud);
    if ($hwflow){
        #print "Using HW Flow Control" if $self->debug;
        $portobj->handshake('rts');
    } elsif ($swflow){
        $portobj->handshake('xoff');
    }
    $portobj->are_match("\r\n","\n","\r");
    
    # Clear the serial port
    $portobj->lookfor;   
    $portobj->lookclear;   

    $self->{COM} = $portobj;
    my $serialize = $self->serialize;
    # Headers on for proper useage
    $self->serialize(0);
    $self->diag->header('off');
    $self->serialize($serialize);
    

    return $portobj;
}

=item $tek->error()

Any command that fails returns undef.  Hit this method to see what
the last error was.

 my $error = $tek->error();

=cut
sub error {
    my $self = shift;
    return $self->{Error};

}

=item $tek->serialize([1|0])

Turns off command serialization.  Internally will do a
'BUSY?' check before sending each command.  See the Programming
Manual for more details.

 # speed up things that we know won't collide
 $tek->serialize(0);
 $tek->command...;
 $tek->command...;
 $tek->serialize(1);

=cut
sub serialize {
    my ($self, $busywait) = @_;

    return $self->{Busywait} unless defined $busywait;
    
    print "[Serialize] Setting to $busywait.\n" if $self->debug;

    $self->{Busywait} = $busywait;
    return $busywait; 
}

=head2 Internally Used Methods

You probably don't want to bother to mess with these, but just in case...

=cut

sub debug { shift->{Debug}; }

sub throw {
    my $self = shift;
    my $error = shift;

    $self->{Error} = $error;

    $error = "[!] $error";
    $error .= "\n" unless $error =~ /\n$/;
    print $error if $self->debug;

    return undef;
}

=item $tek->query(cmd,[argument,argument])

Internally used command that allows you to send command language
directly to the scope.  Use this command to talk to the scope 
yerself.  Will return the results of the given commands.  Use 
set() for commands that don't give results.

Note: The [] in the prototype means 'optional', not 'array reference'.

=cut
sub query {
    my $self = shift;
    my $cmd = shift;
    my @args = @_;

    my $command = "$cmd " . join(',',@args) . "\n";
    print "[Query] $command" if $self->debug;
    
    $self->busywait if $self->{Busywait};

    $self->send($command) or return;
    
    return $self->receive();
}
    
=item $tek->busywait()

Internal use only. Does a 'BUSY?' check and waits for the '0' or '1'.
Internally has timeouts.

=cut
sub busywait {
    my $self = shift;
    my $retries = $self->{Retries};
    # The plus two is to let the receieve timeout first.
    my $timeout = $self->{Timeout} + 2;
    
    print "[Serialize] Waiting for clear\n" if $self->debug;
    
    $self->serial_check;

    # Guilty till proven innocent.
    my $busy = 1;
    for (my $i=0; $i<$retries;$i++){
        alarm($timeout);
        local $SIG{ALRM} = sub { die "Timed out.\n"; };
        eval {
        $self->send("BUSY?\n") or die "Couldn't send.\n";
        my $answer = $self->receive();
        if (!defined $answer){ 
            die("Receive failed.\n");
        } elsif ($answer =~ /0/){
            $busy = 0;
        } elsif ($answer =~ /1/){
            die("Busy Wait - Device responding is Busy.\n");
        }  else {
            die("Got response : $answer, but no busy info.\n");
        }
        };
        if ($@) {
            my $error = $@;
            $self->throw("THS:busywait - $error");
        }
        alarm(0);
        last unless $busy;
        sleep(1) if $busy;
    }
    return $busy;
}

=item $tek->set('cmd',argument,[argument])

Internal use only. Command to set something on the scope.
Returns undef if error, or non-zero if it worked.  Does not
return anything from the scope, use query instead.

=cut
sub set {
    my $self = shift;
    my ($cmd, @args) = @_;

    $self->busywait if $self->{Busywait};

    my $command = "$cmd " . join(',',@args) . "\n";
    print "[Set]   $command" if $self->debug;
    return $self->send($command);
}

=item serial_check()

Internal use only. Checks to make sure the port is not 
CTS blocking or there was a disconnect, etc.

=cut
sub serial_check {
    my $self = shift;
    my $port = $self->{COM};

    my $timeout = $self->{Timeout};
    my $retries = $self->{Retries};

    local $SIG{ALRM} = sub { die "Timed Out.\n"; };
    alarm($timeout);
    eval {
    if ($port->reset_error){
        die "Port Reset Error.\n";
    } else {
    }
    };
    alarm(0);
    if ($@){ 
        $self->throw("THS:serial_check - $@");
        return 0;
    } 

    return 1;
}

=item $tek->receive()

Internal use only.  Waits for a line from the serial port, terminated in a
linefeed.  Used by query()

=cut
sub receive {
    my $self = shift;
    my $port = $self->{COM};
    
    my $timeout = $self->{Timeout};
    my $match = '';
    print "  [rec] Waiting for EOL.\n" if $self->debug;

    unless ($self->serial_check){
        my $error = $self->error;
        die("THS:receive - Serial Port Failed. $error\n");
    }
    
    # Clear Buffer
    $port->lookclear;   
    local $SIG{ALRM} = sub { die "Timed out.\n"; };
    alarm($timeout);
    eval {
    until ($match ne ''){
        select(undef,undef,undef,.1);
        $match = $port->lookfor;
        unless (defined $match){
            die("Error on Serial Port.\n");
        }
        last if $match;
    }
    }; # end eval block 
    alarm(0);
    if ($@) {
        $self->throw("THS:receive - $@");
        return undef;
    }   
    
    my ($found,$end) = $port->lastlook;
    my $st="found : $found , end : $end, match :$match";
    $st =~ s/\n/[nl]/g; 
    $st =~ s/\r/[cr]/g;
    #print "$st\n";

    return $match.$end;
}

=item $tek->send(msg)

Internal use only.  Sends a line to the serial port.

=cut
sub send {
    my $self = shift;
    my $msg = shift;
    my $port = $self->{COM};
    my $length = length($msg);

    print " [send] $length characters.\n" if $self->debug();

    select(undef,undef,undef,.1);
    unless ($self->serial_check){
        my $error = $self->error;
        die("THS:receive - Serial Port Failed. $error\n");
    }
    my $count = $port->write($msg);

    unless ($count == $length){
        return $self->throw("THS::write - Write only wrote $count / $length characters.\n");
    }

    return 1;
}


=head2 Helper Methods

These are for getting around some of the command lanaguage weirdness.

=item $tek->to_num 

Converts the given string to float.  Used to 
convert sci. notation values from scope to perl usable
values.  

=cut
sub to_num {
    my $self = shift;
    my $string = shift;

    return undef unless defined $string;

    return ($string + 0);
}

=item $tek->NR3(number)

Formats a number in socalled 'NR3' format as 
per the programming manual : Floating Point Value with
an Exponent.

No Plus Sign and E is capital, unlike Perl.

=cut
sub NR3 {
    my $self = shift;
    my $num = shift;

    $num = sprintf('%2.1E',$num);
     
    # Bad plus sign, no buiscuit
    $num =~ s/\+//;

    # This turns 1.0E000 to 1.0E0 etc.
    $num =~ s/0+([0-9]+)$/$1/;
    return $num
}

=back

=head1 Command Sub Modules

All the device commands are put into sub catagories to make it all 
more readable, and so I don't have to type as much.

=head2 Aquire Commands

=over

=cut

package Device::Tektronix::Aquire;

# We'll share these constructors and autoload() for all
# the other classes too.
sub new {
    my $proto = shift;
    my $super  = shift;
    my $self = {};
    $self->{SUPER} = $super;
    my $class = ref($proto) || $proto;
    bless ($self, $class);
    return $self;
}

sub AUTOLOAD {
    my $self = shift;
    my $super = $self->{SUPER};
    my $class = ref($self);
    # every class is using Aquire's $autoload
    my $sub = $Device::Tektronix::Aquire::AUTOLOAD;
    return unless defined $sub;
    return if $sub =~ /DESTROY/;
    $sub =~  s/^.*:([^:]+)/$1/;
    if ($super->can($sub)){
        return $super->$sub(@_);
    }
}

=item $tek->aquire->aquire()

Query:  Gives summary of aquire settings.

=cut
sub aquire { return shift->query('ACQ?');}

=item $tek->aquire->mode([mode])

Query:  Shows current Aquire Mode.

Set:    Change Aquisition mode to :

        sample
        peak detect
        average
        envelope

You can truncate these to three letters if you want. (sam/peak/ave/env)

The settings inside the mode is not changed with this command.

=cut
sub mode   { 
    my ($self,$mode) = @_;
    
    return $self->query('ACQ:MOD?') unless defined $mode;
    return $self->throw("Aquire::mode - Must be sample, peakdetect, average or envelope")
        unless $mode =~ /^(sam|peak|ave|env)/i; 
    $mode = 'SAM'  if $mode =~ /^sam/i;
    $mode = 'PEAK' if $mode =~ /^peak/i;
    $mode = 'AVE'  if $mode =~ /^ave/i;
    $mode = 'ENV'  if $mode =~ /^env/i; 
    
    return $self->set('ACQ:MOD',$mode); 
}

=item $tek->aquire->samples()

Query : Returns the number of acquisitions performed by scope in 
current aquire mode.

=cut

sub samples { return shift->query('ACQ:NUMACq'); }

=item $tek->aquire->averages([number of averages])

Query : Returns the number of Averages used for average mode

Set : Turns on Average Aquire Mode and sets the number of averages.

Number must be any number between 2 and 256.

=cut
sub averages { 
    my ($self, $avg) = @_;
    
    return $self->query('ACQ:NUMAVg?') unless defined $avg;

    unless ($avg >= 2 and $avg <= 256 ) {
        return $self->throw("aquire:averages - Number of Averages must be between 2 and 256.");
    }

    $self->aquire->mode('average');

    return $self->set("ACQ:NUMAVg",$avg);
}

=item $tek->aquire->envelope([Number of Envelopes])

Query : Number of Envelopes in use for Envelope Mode

Set : Turns on Envelope mode and sets the number of envelopes 
used.  Number of Envelopes can be any number between 2 and 256.

Set to 0 or 'infinity' for Infinite (free-run) envelope mode.

=cut
sub envelope {
    my ($self, $env) = @_;

    return $self->query('ACQ:NUMENv?') unless defined $env;

    $env = 0 if $env =~ /^(inf)/i;
    unless ($env == 0 or $env >=2 or $env <=256) {
        return $self->throw("aquire:envelope - Number of aquisitions to form envelope must be either 'infinite', or between 2 and 256."); 
    }

    $self->aquire->mode('envelope');
    
    return $self->set("ACQ:NUMENv",$env);
}

=item $tek->aquire->state([state])

Query : What aquire state the scope is in

Set :  Start or Stop data aquisition.

    off/stop    - Like the "hold" Button
    run/start   - Resets the acuisition counter to 0 and starts aquisition.

=cut
sub state {
    my ($self, $state) = @_;

    return $self->query('ACQ:STATE?') unless defined $state;

    unless ($state =~ /^(off|on|run|stop|\d+)$/i){
       return $self->throw("aquire:state - State can be off/on/run/stop"); 
    }
    $state=uc($state);

    return $self->set('ACQ:STATE',$state);
}

=item $tek->aquire->run()

Start / Restart aquisition.

=cut
sub run  { return shift->state('run'); }

=item $tek->aquire->stop()

Stop Aquisition

=cut
sub stop { return shift->state('stop'); }

=item $tek->aquire->stopafter([command])

Query : See what Stopafter mode the scope is in

Set : Have the scope in free-run aquisition mode, or 
have it stop after it has filled its buffers.

    run  - Default behavior
    done - Stop aquisition after "single sequence" of aquisitions.

=cut
sub stopafter {
    my ($self,$state) = @_;
    
    return $self->query('ACQ:STOPA?') unless defined $state;

    unless ($state =~ /^(run|seq|done)/){
        return $self->throw("aquire:stopafter - Must give argument of runstop or done");
    }

    $state = 'RUNST' if $state =~ /^run/i;
    $state = 'SEQ'   if $state =~ /^(seq|done)/i;

    return $self->set('ACQ:STOPA',$state);
}

=back

=head2 Alias Commands

This sub-module includes the Alias Commands
section of the Programming Manual.

Aliases are macro buffers that you can setup and store
complex measurements into.

=over

=cut
package Device::Tektronix::Alias;
*Device::Tektronix::Alias::new      = \&Device::Tektronix::Aquire::new;
*Device::Tektronix::Alias::AUTOLOAD = \&Device::Tektronix::Aquire::AUTOLOAD;

=item $tek->alias->alias()

Query : See if Aliases are on

Set : Turn Aliases off and on.

=cut
sub alias {
    my ($self,$set) = @_;
    
    return $self->query('ALIAS?') unless (defined $set);

    unless ($set =~ /^(on|off|\d+)$/i) {
        return $self->throw("Alias:alias - Use on/off/0/1 to toggle aliases.");
    }

    $set=uc($set);
    return $self->set('ALIAS', $set);
}

=item $tek->alias->list()

Query : Lists different names set

=cut
sub list {

}


=back

=head2 Diagnostic Commands

This sub-module includes the following sections
from the Programming Manual:

=over

=item Status and Error

=item Misc

=item Calibration

=item Diagnostics

=back

=over

=cut
package Device::Tektronix::Diag;
*Device::Tektronix::Diag::new      = \&Device::Tektronix::Aquire::new;
*Device::Tektronix::Diag::AUTOLOAD = \&Device::Tektronix::Aquire::AUTOLOAD;

=item $tek->diag->header([on|off])

Query : See if Header is on (any query will tell you this)

Set : Turn Header on or off.   Takes on/off/0/1 as arg.

Headers are turned off by connect()

Turning headers on might break some internal functions. Maybe.

=cut
sub header {
    my ($self, $header) = @_;

    return $self->query('HEADer?') unless (defined $header);
    unless ($header =~ /^(on|off|\d+)$/i){
        return $self->throw("Diag:Header - Argument needs to be on/off/0/1\n");
    }
    
    print "[Header] Turning headers $header\n" if $self->debug;

    return  $self->set('HEAD',uc($header));
}

=item $tek->diag->id()

Query : ID String

=cut
sub id {
    my $self = shift;
        
    return $self->query('ID?');
    
}

=item $tek->diag->lock([all|none])

Query : See if front panel is locked

Set : Turn lock on or off.   Takes all/none as argument.

The On button is never locked.

=cut
sub lock {
    my ($self, $lock) = @_;

    return $self->query('LOC?') unless (defined $lock);

    unless ($lock =~ /^(all|none)$/i){
        return $self->throw("Diag:Lock - Argument needs to be all or none\n");
    }
    
    return  $self->set('LOC',uc($lock));
}

=back

=head2

=over

=cut


=back

=head2

=over

=cut

package Device::Tektronix::Cursor;
*Device::Tektronix::Cursor::new      = \&Device::Tektronix::Aquire::new;
*Device::Tektronix::Cursor::AUTOLOAD = \&Device::Tektronix::Aquire::AUTOLOAD;

=back

=head2 

=over

=cut

package Device::Tektronix::Display;
*Device::Tektronix::Display::new      = \&Device::Tektronix::Aquire::new;
*Device::Tektronix::Display::AUTOLOAD = \&Device::Tektronix::Aquire::AUTOLOAD;

=back

=head2

=over

=cut

package Device::Tektronix::Print;
*Device::Tektronix::Print::new      = \&Device::Tektronix::Aquire::new;
*Device::Tektronix::Print::AUTOLOAD = \&Device::Tektronix::Aquire::AUTOLOAD;

=back

=head2

=over

=cut

package Device::Tektronix::Harmonics;
*Device::Tektronix::Harmonics::new      = \&Device::Tektronix::Aquire::new;
*Device::Tektronix::Harmonics::AUTOLOAD = \&Device::Tektronix::Aquire::AUTOLOAD;

=back

=head2 Time Settings (Horizontal Commands)

=over

=cut

package Device::Tektronix::Time;
*Device::Tektronix::Time::new      = \&Device::Tektronix::Aquire::new;
*Device::Tektronix::Time::AUTOLOAD = \&Device::Tektronix::Aquire::AUTOLOAD;

=item scale

Query : Timebase returned as number (not string)

Set : Number from 5ns to 5s with 1-2-5 granularity.

=cut

sub scale {
    my ($self,$scale) = @_;

    return $self->to_num($self->query('HOR:MAI:SCA?')) unless defined $scale; 
    
    return $self->set('HOR:MAI:SCA',$self->NR3($scale));
}

=back

=head2 Channel Commands (Vertical Commands)

=over

=cut

package Device::Tektronix::Channel;
*Device::Tektronix::Channel::new      = \&Device::Tektronix::Aquire::new;
*Device::Tektronix::Channel::AUTOLOAD = \&Device::Tektronix::Aquire::AUTOLOAD;

=item $tek->channel->status( channel_no )

Query : Get Channel info for channel_no

=cut
sub status {
    my ($self,$channel) = @_;

    return $self->query("CH${channel}?");
}

=item $tek->channel->probe( channel_no )

Query : Get Probe info for channel_no

=cut
sub probe {
    my ($self,$channel) = @_;

    return $self->query("CH${channel}:PRO?");
}

=item $tek->channel->bandwidth( channel_no [20/full] )

Query : Get Bandwidth setting for channel 

Set : Set Bandwidth

=cut
sub bandwidth {
    my ($self, $channel, $bw) = @_;
    return $self->query("CH${channel}:BAN?") unless defined $bw;
    
    unless ($bw =~ /^(twenty|20|full)/i){
        return $self->throw("Channel:Bandwidth - Must be 20/full\n");
    }

    $bw = 'TWE' if $bw =~ /(20|twe)/;
    $bw = 'FUL'   if $bw =~ /full/;

    return $self->set("CH${channel}:BAN",$bw);
}


=item $tek->channel->coupling( channel_no [ac|dc|gnd] )

Query : Get coupling setting for channel 

Set : Set coupling

=cut
sub coupling {
    my ($self, $channel, $couple) = @_;
    return $self->query("CH${channel}:COUPL?") unless defined $couple;
    
    unless ($couple =~ /^(ac|dc|gnd)/i){
        return $self->throw("Channel:coupling - Must be ac/dc/gnd\n");
    }

    return $self->set("CH${channel}:COUPL",$couple);
}

=item $tek->channel->invert( channel_no [on|off])

Query : Get invert setting for channel 

Set : Set invert

    on
    off

=cut
sub invert {
    my ($self, $channel, $invert) = @_;
    return $self->query("CH${channel}:INV?") unless defined $invert;
    
    unless ($invert =~ /^(on|off|0|1)$/){
        return $self->throw("Channel:invert - Must be ac/dc/gnd\n");
    }

    return $self->set("CH${channel}:INV",uc($invert));
}

=item $tek->channel->position( channel_no [delta_divs])

Query : Get position setting for channel 

Set : Move position by given number of divisions

Value can be +- 10 divs according to my scope, but +- 5 according to the
programming menu.

=cut
sub position {
    my ($self, $channel, $position) = @_;
    return $self->query("CH${channel}:POS?") unless defined $position;
    
    unless ($position >= -10 and $position <= 10){
        return $self->throw("Channel:Position - Must be +- 10 divs max\n");
    }
    return $self->set("CH${channel}:POS",$self->NR3($position));
}

=item $tek->channel->units( channel_no [volts|amps])

Query : Get units setting for channel 

Set : Set the probe type to voltage or current probe

    volts
    amps

=cut
sub units {
    my ($self, $channel, $units) = @_;
    return $self->query("CH${channel}:PROBe:UNITs?") unless defined $units;
    
    $units = lc($units);
    unless ($units =~ /(volts|amps)/){
        # No femptoseimens here.
        return $self->throw("Channel:units - Must be volts or amps.\n");
    }

    $units = 'VOL' if $units eq 'volts';
    $units = 'AMPS'  if $units eq 'amps';
    return $self->set("CH${channel}:PROB:UNIT",$units);
}

=item $tek->channel->probescale( channel_no [1|10|50|100|500|1000])

Query : Get Voltage Probe Attenuation Factor setting for channel 

Set : Set the probe type 

    1  - 1X
    10 - 10X
    ...

=cut
sub probescale {
    my ($self, $channel, $probescale) = @_;
    return $self->query("CH${channel}:PROB:VOLTSCALE?") unless defined $probescale;
    
    $probescale = lc($probescale);
    unless ($probescale =~ /^(1|10|50|100|500|1000)$/){
        # No femptoseimens here.
        return $self->throw("Channel:probescale - bad value ($probescale).\n");
    }

    return $self->set("CH${channel}:PROB:VOLTSCALE",$self->NR3($probescale));
}

=item $tek->channel->scale( channel_no [0.005 - 50000])

Query : Get Vertical Gain of Channel.   Same as VOLTS/DIV Button.

Set : Set the Volts per division

Depending on the Probe multiplier in Use, only certain settings work.

 1x Probe  - [ .005 .01 .02 .05 .1  .2  .5  1  2  5  10  20  50  ] 
 10x Probe - [ 0.05 0.1 0.2 0.5 1   2   5   10 20 50 100 200 500 ]
 1000x     - [ 5    10  20  50  100 200 500 1k 2k 5k 10k 20k 50k ]

You can figure out the rest.  You might just want to make sure / input what type
of probe is on there from the user before setting the scale.  It's safe to
assume 10x, unless using a THS720P.

=cut
sub scale {
    my ($self, $channel, $scale) = @_;
    return $self->query("CH${channel}:SCALE?") unless defined $scale;
    
    unless ($scale >= 0.005 and $scale <= 50_000) {
        return $self->throw("Channel:scale - Out of Range ($scale).\n");
    }

    return $self->set("CH${channel}:SCALE",$self->NR3($scale));
}

=back

=head2

=over

=cut

package Device::Tektronix::Measure;
*Device::Tektronix::Measure::new      = \&Device::Tektronix::Aquire::new;
*Device::Tektronix::Measure::AUTOLOAD = \&Device::Tektronix::Aquire::AUTOLOAD;

=back

=head2

=over

=cut

package Device::Tektronix::Serial;
*Device::Tektronix::Serial::new      = \&Device::Tektronix::Aquire::new;
*Device::Tektronix::Serial::AUTOLOAD = \&Device::Tektronix::Aquire::AUTOLOAD;

=back

=head2 Trigger Commands

=over

=cut

package Device::Tektronix::Trigger;
*Device::Tektronix::Trigger::new      = \&Device::Tektronix::Aquire::new;
*Device::Tektronix::Trigger::AUTOLOAD = \&Device::Tektronix::Aquire::AUTOLOAD;

=item $tek->trigger->trigger([go])

Query : Get current Trigger settings

Set   : Give any argument to force a trigger event

=cut
sub trigger{
    my ($self,$trigger) = @_;
    
    return $self->query('TRIG?') unless defined $trigger;

    return $self->set('TRIG','FORC');
}

=item $tek->trigger->auto_set()

Set : This one does the same as hitting the "Set Level to 50%" Button.

=cut
sub auto_set {
    my $self = shift;

    return $self->set('TRIG:MAI','SETL');
}

=item $tek->trigger->main()

Query : Gives info about the main trigger.

=cut
sub main {
    my $self = shift;
    
    return $self->query('TRIG:MAI:EDGE?');
}

=item $tek->trigger->slope([falling|rising])

Query : Duh

Set   : Duh

=cut
sub slope {
    my ($self,$slope) = @_;

    return $self->query('TRIG:MAI:EDGE:SLO?') unless defined $slope;

    $slope = 'FALL'  if $slope =~ /fal/i;
    $slope = 'RIS'   if $slope =~ /ris/i;

    return $self->set('TRIG:MAI:EDGE:SLO',$slope);
}

=item $tek->trigger->source([ch1|ch2|dmm])

Query : Source of trigger

Set   : Source of trigger

    ch1
    ch2
    dmm (external)

=cut
sub source {
    my ($self,$source) = @_;

    return $self->query('TRIG:MAI:EDGE:SOU?') unless defined $source;

    $source = 'CH1' if $source =~ /1/;
    $source = 'CH2' if $source =~ /2/;
    $source = 'EXT' if $source =~ /(dmm|ext)/i;

    return $self->set('TRIGGER:MAIN:EDGE:SOURCE',$source);
}

=item $tek->trigger->couple([])

Query : Get current mode for trigger coupling

Set :

    dc    - DC Coupling
    hf    - High Freq Rejection on DC Signal
    lf    - Low Freq Rejection  on AC Signal
    noise - DC Low sensitivity.  Requires more amplitude...

=cut
sub couple {
    my ($self,$couple) = @_;    

    return $self->query('TRIG:MAI:EDGE:COUP?') unless defined $couple;

    $couple = 'DC' if $couple =~ /dc/i;
    $couple = 'HFR' if $couple =~ /hf/i;
    $couple = 'LFR' if $couple =~ /lf/i;
    $couple = 'NOISE' if $couple =~ /noise/i;

    return $self->set('TRIG:MAI:EDGE:COUP',$couple);
}

# TODO - Holdoff

=item level([some voltage|ttl|ecl])

Query : Returns the trigger level setting

Set : Set the trigger level

    ECL - Scope preset of -1.3V
    TTL - Scope preset of 1.4V
    some voltage - Whatever you want.

Note: In DMM triggering mode, you have only two choices, either
0.2 or 2.0

=cut
sub level {
    my ($self,$level) = @_;

    return $self->query('TRIG:MAI:LEV?') unless defined $level;

    $level = 'ECL' if $level =~ /ecl/;
    $level = 'TTL' if $level =~ /ttl/;
    $level = $self->NR3($level) unless $level =~ /(ECL|TTL)/;

    $self->set('TRIG:MAI:LEV',$level);
}
=back

=head2 Waveforms Menus

Includes Data, Curve and Waveform commands

=over

=cut

package Device::Tektronix::Waveform;
*Device::Tektronix::Waveform::new      = \&Device::Tektronix::Aquire::new;
*Device::Tektronix::Waveform::AUTOLOAD = \&Device::Tektronix::Aquire::AUTOLOAD;

=item $tek->waveform->data([init|snap)

Query : Lists the waveform data parameters

Set : Sets the waveform parameters 

    init - Factory defaults
    snap - USe vertical bars for start and stop

=cut
sub data {
    my ($self, $data) = @_;

    return $self->query('DATa?') unless (defined $data);

    unless ($data =~ /^(init|sna)/i){
        return $self->throw("Waveform:Data - Argument needs to be init or snap\n");
    }
    
    return  $self->set('DATa',uc($data));
}

=item $tek->waveform->dest([refa|refb|sto1|...|sto10])

Query : Lists the waveform destination register

Set : Sets the destination register

    refa - Reference Waveform A
    refb - Reference Waveform B
    sto1 - Storage 1 (not usable from scope interface)
    ...
    sto10- Storage 10

=cut
sub dest {
    my ($self, $dest) = @_;

    return $self->query('DATa:DESTINATION?') unless (defined $dest);

    unless ($dest =~ /^(refa|refb|sto[0-9]+)/i){
        return $self->throw("Waveform:dest - Argument needs to be refa/refb/sto1...\n");
    }
    
    return  $self->set('DATa:DESTINATION',uc($dest));
}

=item $tek->waveform->encoding([])

Query : Lists the waveform encoding

Set : Sets the waveform encoding

    ascii - plain text, signed integer
    smsb  - signed, MSB     (-128...127) or (-32768...32767)
    umsb  - unsigned, MSB   (0...255)    or (0     ...65535)
    slsb  - signed, LSB
    ulsb  - unsigned MSB

=cut
sub encoding {
    my ($self, $encoding) = @_;

    return $self->query('DAT:ENC?') unless (defined $encoding);

    $encoding = lc($encoding);
    unless ($encoding =~ /^(ascii|smsb|umsb|slsb|ulsb)$/){
        return $self->throw("Waveform:encoding - Argument needs to be ascii/smsb/umsb/slsb/ulsb\n");
    }
    
    $self->{Encoding}=$encoding;
    $encoding = 'ASCI' if $encoding eq  'ascii';
    $encoding = 'RIB' if $encoding eq 'smsb';
    $encoding = 'RPB' if $encoding eq 'umsb';
    $encoding = 'SRI' if $encoding eq 'slsb';
    $encoding = 'SRP' if $encoding eq 'ulsb';

    return  $self->set('DAT:ENC',$encoding);
}

=item $tek->waveform->source([init|snap])

Query : Lists the waveform source

Set : Sets the waveform Source Channel

    ch1
    ch2
    math
    refa
    refb
    sto0
    ...
    sto10

You can specify multiple sources. Data always comes back in order
above.  Multiple source data from curve() is separated by a comma.

=cut
sub source {
    my ($self, @sources) = @_;

    return $self->query('DAT:SOU?') unless (scalar @sources);

    unless ($sources[0] =~ /^(ch1|ch2|math|refa|refb|sto\d+)$/){
        return $self->throw("Waveform:source - Argument needs to be channel source name\n");
    }
    
    return  $self->set('DAT:SOU',map uc, @sources);
}

=item $tek->waveform->start([1...2500])

Query : Lists the waveform starting point

Set : Sets the waveform starting point

    1 - 2500  for scope mode
    1 - 480   for DMM mode

=cut
sub start {
    my ($self, $start) = @_;

    return $self->query('DAT:STAR?') unless (defined $start);

    unless ($start =~ /^[1-9][0-9]*$/){
        return $self->throw("Waveform:Start - Argument needs to be 1-2500 / 1-480\n");
    }
    
    $self->{START}=$start;
    
    return  $self->set('DAT:STAR',$start);
}

=item $tek->waveform->stop([1...2500])

Query : Lists the waveform stopping point

Set : Sets the waveform stopping point

    1 - 2500  for scope mode
    1 - 480   for DMM mode

=cut
sub stop {
    my ($self, $stop) = @_;

    return $self->query('DAT:STOP?') unless (defined $stop);

    unless ($stop =~ /^[1-9][0-9]*$/){
        return $self->throw("Waveform:stop - Argument needs to be 1-2500 / 1-480\n");
    }
    
    $self->{STOP}=$stop;
    
    return  $self->set('DAT:STOP',$stop);
}

=item $tek->waveform->width([8|16])

Query : Lists the bytes per sample (1 or 2)

Set : Sets the waveform widthping point

    8  - 8it (1 byte)
    16 - 16bit (2 bytes)

Note that the scope samples at 8-bit.  16bit is good for
Averaged waveforms.  The MSB is simply 0 for the rest.

=cut
sub width {
    my ($self, $width) = @_;

    return $self->query('DAT:WID?') unless (defined $width);

    unless ($width =~ /^(8|16)$/){
        return $self->throw("Waveform:width - Argument needs to be 8 or 16\n");
    }
    
    $width /=8;
    $self->{WIDTH}=$width;

    return  $self->set('DAT:WID',$width);
}

=item $tek->waveform->get()

Query : Get waveform data from scope.

=cut
sub get {
    my $self = shift;

    my $start = $self->{START};
    my $stop  = $self->{STOP};
    my $width = $self->{WIDTH};

    unless (defined $start and defined $start and defined $width){
        return $self->throw("Waveform:get - You must run start() stop() and width() before you can aquire a waveform.\n");
    }

    my $bytes = ($stop-$start)*$width;

    $self->busywait() if $self->serialize;
    print "[Waveform:get] Getting $bytes bytes.\n" if $self->debug;

    #my $data = $self->query("CURV?");

    $self->send("CURV?\n");
    my $port = $self->{SUPER}->{COM};
    $port->read_interval(0);
    $port->read_const_time(5000);
    # Extra hundred for header
    my ($count,$data) = $port->read($bytes+100);
    
    return undef unless defined $data;

    unless ($count >= $bytes){
        $self->throw("THS:get - Received only $count bytes. Expected $bytes or more.\n");
        return undef;
    }
    
    # Data will have Tektronix wave header with it, want only the data.
    my $header = substr($data,0,$count-$bytes);
    print "[Waveform:Get] Got $count bytes.  Header : $header\n" if $self->debug;
    $data = substr($data,$count-$bytes,$bytes);
    
    return $data;
}
=back

=head2

=over

=cut

package Device::Tektronix::DMM;
*Device::Tektronix::DMM::new      = \&Device::Tektronix::Aquire::new;
*Device::Tektronix::DMM::AUTOLOAD = \&Device::Tektronix::Aquire::AUTOLOAD;

1;

=back

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2003 by Max Baker

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.1 or,
at your option, any later version of Perl 5 you may have available.

=cut
